@@ -29,6 +29,7 @@ class Truss():
         self.plate_dir = f'{db_dir}/plates_sorting'
         self.status_file = f'{db_dir}/status/{name}.json'
         self.params_file = f'{db_dir}/params/{name}.json'
+        self.stacks_file = f'{db_dir}/stacks/{name}.json'
 
         self.server = snap7.server.Server()
         self.dbs = dict()
@@ -86,7 +87,7 @@ class Truss():
             x_end = unpack('!f',pack('4b',self.db0[70],self.db0[71],self.db0[72],self.db0[73]))[0]
             x_start = unpack('!f',pack('4b',self.db1[18],self.db1[19],self.db1[20],self.db1[21]))[0]
 
-        for interval in np.linspace(x_start,x_end,int(abs(x_start - x_end) / 200)):
+        for interval in np.linspace(x_start,x_end,int(abs(x_start - x_end) / 200)+1):
             if index == 0: self.db1[2],self.db1[3],self.db1[4],self.db1[5] = pack('!f',interval)
             elif index == 1: self.db1[18],self.db1[19],self.db1[20],self.db1[21] = pack('!f',interval)
             self.x[index] = interval
@@ -105,7 +106,7 @@ class Truss():
         elif index == 1:
             end = unpack('!f',pack('4b',self.db0[74],self.db0[75],self.db0[76],self.db0[77]))[0]
             start = unpack('!f',pack('4b',self.db1[22],self.db1[23],self.db1[24],self.db1[25]))[0]
-        for interval in np.linspace(start,end,int(abs(start - end) / 200)):
+        for interval in np.linspace(start,end,int(abs(start - end) / 200)+1):
             if index == 0: self.db1[6],self.db1[7],self.db1[8],self.db1[9] = pack('!f',interval)
             elif index == 1: self.db1[22],self.db1[23],self.db1[24],self.db1[25] = pack('!f',interval)
             self.y[index] = interval
@@ -118,7 +119,7 @@ class Truss():
         elif index == 1:
             end = unpack('!f',pack('4b',self.db0[78],self.db0[79],self.db0[80],self.db0[81]))[0]
             start = unpack('!f',pack('4b',self.db1[26],self.db1[27],self.db1[28],self.db1[29]))[0]
-        for interval in np.linspace(start,end,int(abs(start - end) / 100)):
+        for interval in np.linspace(start,end,int(abs(start - end) / 100)+1):
             if index == 0: self.db1[10],self.db1[11],self.db1[12],self.db1[13] = pack('!f',interval)
             elif index == 1: self.db1[26],self.db1[27],self.db1[28],self.db1[29] = pack('!f',interval)
             self.z[index] = interval
@@ -130,7 +131,7 @@ class Truss():
         elif index == 1:
             end = unpack('!f',pack('4b',self.db0[82],self.db0[83],self.db0[84],self.db0[85]))[0]
             start = unpack('!f',pack('4b',self.db1[30],self.db1[31],self.db1[32],self.db1[33]))[0]
-        for interval in np.linspace(start,end,int(abs(start - end) / 50)):
+        for interval in np.linspace(start,end,int(abs(start - end) / 10)+1):
             if index == 0: self.db1[14],self.db1[15],self.db1[16],self.db1[17] = pack('!f',interval)
             elif index == 1: self.db1[30],self.db1[31],self.db1[32],self.db1[33] = pack('!f',interval)
             self.r[index] = interval
@@ -182,7 +183,7 @@ class Truss():
                 self.z[index] = interval
                 await asyncio.sleep(0.01)
         async def r(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 50)+1):
+            for interval in np.linspace(start,end,int(abs(start - end) / 10)+1):
                 if index == 0: self.db1[14],self.db1[15],self.db1[16],self.db1[17] = pack('!f',interval)
                 elif index == 1: self.db1[30],self.db1[31],self.db1[32],self.db1[33] = pack('!f',interval)
                 self.r[index] = interval
@@ -306,6 +307,7 @@ class Truss():
 
             self.status = dson(self.status_file)
             self.params = dson(self.params_file)
+            self.stacks = dson(self.stacks_file)
             if 'plate_name' not in self.status: continue
             plate_name = self.status['plate_name']
             self.plate = dson(f'{self.plate_dir}/{plate_name}.json')
@@ -340,19 +342,41 @@ class Truss():
             self.view.text(plate_p1[0],plate_p1[1],plate_p1[2],f'{plate_p1} {plate_degree}°')
 
             #画托盘
-            drop_pos = self.params['sorting.pallet_pos']
-            pallet_size = self.params['sorting.pallet_size']
+            for drop_pos,pallet_size in [
+                    (self.params['placing.pallet_pos'],self.params['placing.pallet_size']),
+                    (self.params['placing.pallets.left.pos'],self.params['placing.pallets.left.size']),
+                    (self.params['placing.pallets.middle.pos'],self.params['placing.pallets.middle.size']),
+                    (self.params['placing.pallets.right.pos'],self.params['placing.pallets.right.size'])]:
+                if not drop_pos: continue
+                p0 = drop_pos
+                p1 = drop_pos[0], drop_pos[1] + pallet_size[1], drop_pos[2]
+                p2 = drop_pos[0] + pallet_size[0], drop_pos[1] + pallet_size[1], drop_pos[2]
+                p3 = drop_pos[0] + pallet_size[0], drop_pos[1], drop_pos[2]
+                self.rectangle(p0,p1,p2,p3)
+                self.view.text(p3[0],p3[1],p3[2],s=str(drop_pos))
             
-            p0 = drop_pos
-            p1 = drop_pos[0], drop_pos[1] + pallet_size[1], drop_pos[2]
-            p2 = drop_pos[0] + pallet_size[0], drop_pos[1] + pallet_size[1], drop_pos[2]
-            p3 = drop_pos[0] + pallet_size[0], drop_pos[1], drop_pos[2]
-            self.rectangle(p0,p1,p2,p3)
-            self.view.text(p3[0],p3[1],p3[2],s=str(drop_pos))
+            #画码垛
+            for pallet_name,stacks in self.stacks.value.items():
+                pos = self.params[f'placing.pallets.{pallet_name}.pos']
+                for stack in stacks:
+                    for name,offset,part_size,kind,real_size in stack:
+                        p0 = pos[0] + offset[0], pos[1] + offset[1], pos[2] + offset[2]
+                        p1 = pos[0] + offset[0], pos[1] + offset[1] + part_size[1], pos[2] + offset[2]
+                        p2 = pos[0] + offset[0] + part_size[0], pos[1] + offset[1] + part_size[1], pos[2] + offset[2]
+                        p3 = pos[0] + offset[0] + part_size[0], pos[1] + offset[1], pos[2] + offset[2]
+                        self.rectangle(p0,p1,p2,p3)
 
-            if self.plate['finished']:
-                drop_positions = self.plate['tiling']
-            else:
+                        p0 = pos[0] + offset[0], pos[1] + offset[1], pos[2] + offset[2]
+                        p1 = pos[0] + offset[0], pos[1] + offset[1] + real_size[1], pos[2] + offset[2]
+                        p2 = pos[0] + offset[0] + real_size[0], pos[1] + offset[1] + real_size[1], pos[2] + offset[2]
+                        p3 = pos[0] + offset[0] + real_size[0], pos[1] + offset[1], pos[2] + offset[2]
+                        self.rectangle(p0,p1,p2,p3)
+                        # self.view.text(p3[0],p3[1],p3[2],s=str(name))
+
+            #画平铺
+            
+            drop_positions = []
+            if 'tiling' in self.plate: 
                 drop_positions = self.plate['tiling']
 
             for part in self.plate['parts']:
@@ -412,8 +436,8 @@ class Truss():
                 if id not in drop_positions:continue
                 if not part['picked']: continue
 
-                drop_pos,pallet_name = drop_positions[id]
-                drop_pos_offset = self.params['sorting.pallet_pos']
+                drop_pos,_,_,pallet_name = drop_positions[id]
+                drop_pos_offset = self.params['placing.pallet_pos']
                 drop_pos = drop_pos_offset[0] + drop_pos[0],drop_pos_offset[1] + drop_pos[1],drop_pos_offset[2] + drop_pos[2]
 
                 if 0 == handle:
@@ -475,7 +499,10 @@ class Truss():
                     p3 = [box_left,box_bottom,drop_pos[2]]
                     
                     self.rectangle(p0, p1, p2, p3)
-        except:pass
+        except:
+            import traceback
+            traceback.print_exc()
+            pass
                     
         self.view.plot([self.x[0],self.x[0]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red',linewidth=1)
         self.view.plot([self.x[1],self.x[1]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red',linewidth=1)
